--[[---------------------------------------------------------------------------\
| ||\\    //||       /|¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯\ |
| || \\  // ||  (o_ / |                  SUPPLEMENTARY FILE                  | |
| ||  \\//  ||  //\/  |                         ----                         | |
| ||   \/   ||  V_/_  |                    DAY NIGHT CYCLE                   | |
| ||        ||        |‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗‗/ |
\---------------------------------------------------------------------------]]--

local dayNight = ...

--[[--------------------------- PREFABS & MATERIALS ---------------------------]]--

--[[---------------------------- CUSTOM COMPONENTS ----------------------------]]--

local COMP_DAY_NIGHT_CYCLE = {
	TypeName = "COMP_DAY_NIGHT_CYCLE",
	ParentType = "COMPONENT",
	Properties = {
        --{ Name = "Timer", Type = "float", Default = 420 },
        { Name = "Shader", Type = "GAME_OBJECT", Default = nil },
        { Name = "IsManuallySet", Type = "boolean", Default = false }
    }
}

function COMP_DAY_NIGHT_CYCLE:create()
    self.sun = nil
end

function COMP_DAY_NIGHT_CYCLE:init()
    self.sun = self:getLevel():find("Sun")
    local camera = self:getOwner()
    self.Shader = self:getLevel():find("DayNightShader")
    if self.Shader == nil then
        self.Shader = self:getLevel():createObject("PREFAB_DAY_NIGHT_SHADER", camera:getGlobalPosition(), camera.Orientation)
        self.Shader:setParent(camera, true)
    end
end

function COMP_DAY_NIGHT_CYCLE:setManual(bool)
    if bool == true then
        self.IsManuallySet = true
        self.Shader.Active = false
        local compSunController = self:getLevel():find("COMP_SUN_CONTROLLER")
        compSunController:resetSun()
    else
        self.IsManuallySet = false
        self.Shader.Active = true
        local compSunController = self:getLevel():find("COMP_SUN_CONTROLLER")
        compSunController:resetSun()
    end
end

function COMP_DAY_NIGHT_CYCLE:update()
    if not self.IsManuallySet then
        local dt = self:getLevel():getDeltaTime()
        
        self.sun:rotateY(0.015*dt)
        
        if self.Shader ~= nil then
            self.Shader:rotateLocalX(0.015*dt)
        end
    end
end

dayNight:registerClass(COMP_DAY_NIGHT_CYCLE)

--[[--------------------- ASSET PROCESSOR & NODE HANDLING ---------------------]]--
--[[--------------------------- COMPONENT ASSIGNMENT --------------------------]]--
--[[-------------------------------- COLLIDERS --------------------------------]]--

dayNight:registerPrefabComponent("PREFAB_GAME_CAMERA", {
    DataType = "COMP_DAY_NIGHT_CYCLE"
})

--[[------------------------ BUILDINGS & BUILDING PARTS -----------------------]]--

--[[------------------------- JOBS & BUILDING FUNCTIONS -----------------------]]--

--[[----------------------------- BEHAVIOUR TREES -----------------------------]]--